@@ -1,7 +1,10 @@
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
@@ -10,6 +13,7 @@
 #include "str_convert.h"
 #include "file_util.h"
 #include "elist.h"
+#include "chrono.h"
 #include "gtk_util.h"
 #include "mru.h"
 #include "prefs.h"
@@ -17,13 +21,17 @@
 #include "message_box.h"
 #include "main_win.h"
 #include "status_bar.h"
-#include "progress_dlg.h"
 #include "rename_dlg.h"
+#include "progress_dlg.h"
+#include "scan_progress_dlg.h"
 #include "edit_tab.h"
 
 #include "file_list.h"
 
 
+#define PROGRESS_DLG_DELAY  1000
+
+
 /* widgets */
 static GtkWindow *w_main = NULL;
 
@@ -37,6 +45,8 @@ static GtkListStore *store_files = NULL;
 static GtkLabel *lab_file_count = NULL;
 static GtkMenu *menu_file_list = NULL;
 static GtkMenuItem *m_ctx_manual_rename = NULL;
+static GtkMenuItem *m_ctx_delete = NULL;
+static GtkMenuItem *m_ctx_unselect_all = NULL;
 
 /* icons */
 static GdkPixbuf *pix_file;
@@ -46,7 +56,9 @@ static GdkPixbuf *pix_folder;
 static GString *working_dir = NULL;
 static GString *working_dir_utf8 = NULL;
 static GEList *file_list = NULL;
-static gchar *editing_file;
+static gchar *editing_file = NULL;
+static gboolean progress_dlg_visible;
+static chrono_t progress_dlg_time;
 
 /* preferences */
 static gboolean *recurse;
@@ -101,14 +113,16 @@ static void update_file_count()
 	str = g_string_sized_new(60);
 
 	if (total == 0)
-		g_string_assign(str, "No files found");
+		g_string_assign(str, _("No files found"));
 	else if (total == 1)
-		g_string_assign(str, "1 file found");
+		g_string_assign(str, _("1 file found"));
 	else
-		g_string_printf(str, "%i files found", total);
+		g_string_printf(str, _("%i files found"), total);
 
-	if (selected > 0)
-		g_string_append_printf(str, " (%i selected)", selected);
+	if (selected == 1)
+		g_string_append(str, _(" (1 selected)"));
+	else if (selected > 1)
+		g_string_append_printf(str, _(" (%i selected)"), selected);
 
 	gtk_label_set_text(lab_file_count, str->str);
 
@@ -124,20 +138,20 @@ static void update_tree_view(const GEList *file_list)
 	GtkTreeIter tree_iter;
 	GList *i;
 
-	gtk_list_store_clear(store_files);
-
 	style = gtk_widget_get_style(GTK_WIDGET(tv_files));
 	if (style == NULL)
 		g_error("Couldn't get style for widget tv_files");
 
 	gtk_tree_view_set_model(tv_files, NULL);
 
+	gtk_list_store_clear(store_files);
+
 	i = g_elist_first(file_list);
 	while (i) {
 		if (!fu_compare_file_paths(last_file, i->data)) {
 			char *dirname = g_dirname(i->data);
 			aux = str_filename_to_utf8(dirname,
-						   "(dir name could not be converted to UTF8)");
+						   _("(dir name could not be converted to UTF8)"));
 			gtk_list_store_append(store_files, &tree_iter);
 			gtk_list_store_set(store_files, &tree_iter, 
 					   0, pix_folder,
@@ -150,7 +164,7 @@ static void update_tree_view(const GEList *file_list)
 			g_free(aux);
 		}
 		aux = str_filename_to_utf8(fu_last_n_path_components(i->data, 1),
-					   "(file name could not be converted to UTF8)");
+					   _("(file name could not be converted to UTF8)"));
 		gtk_list_store_append(store_files, &tree_iter);
 		gtk_list_store_set(store_files, &tree_iter, 
 				   0, pix_file,
@@ -179,8 +193,11 @@ static void clear_tree_view()
 
 static void popup_tree_view_menu()
 {
-	int selected = gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(tv_files));
-	gtk_widget_set_sensitive(GTK_WIDGET(m_ctx_manual_rename), selected == 1);
+	int sel_count = gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(tv_files));
+
+	gtk_widget_set_sensitive(GTK_WIDGET(m_ctx_manual_rename), sel_count == 1);
+	gtk_widget_set_sensitive(GTK_WIDGET(m_ctx_delete),        sel_count > 0);
+	gtk_widget_set_sensitive(GTK_WIDGET(m_ctx_unselect_all),  sel_count > 0);
 	
 	gtk_menu_popup(menu_file_list, NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time());
 }
@@ -192,37 +209,37 @@ static void delete_files(const GEList *file_list)
 	char *temp_utf8;
 	int count_deleted = 0;
 
-	pd_start("Deleting Files");
+	pd_start(_("Deleting Files"));
 
 	for (iter = g_elist_first(file_list); iter; iter = g_list_next(iter)) {
 		/* flush pending gtk operations so the UI doesn't freeze */
 		pd_scroll_to_bottom();
 		while (gtk_events_pending()) gtk_main_iteration();
 		if (pd_stop_requested()) {
-			pd_printf(PD_ICON_WARN, "Operation stopped at user's request");
+			pd_printf(PD_ICON_WARN, _("Operation stopped at user's request"));
 			break;
 		}
 
-		temp_utf8 = str_filename_to_utf8(iter->data, "(UTF8 conversion error)");
+		temp_utf8 = str_filename_to_utf8(iter->data, _("(UTF8 conversion error)"));
 
 		if (unlink(iter->data) == 0) {
 			count_deleted++;
-			pd_printf(PD_ICON_OK, "Deleted file \"%s\"", temp_utf8);
+			pd_printf(PD_ICON_OK, _("Deleted file \"%s\""), temp_utf8);
 		}
 		else {
 			int save_errno = errno;
-			pd_printf(PD_ICON_FAIL, "Error deleting file \"%s\"", temp_utf8);
-			pd_printf(PD_ICON_NONE, "%s (errno %d)", strerror(save_errno), save_errno);
+			pd_printf(PD_ICON_FAIL, _("Error deleting file \"%s\""), temp_utf8);
+			pd_printf(PD_ICON_NONE, "%s (%d)", strerror(save_errno), save_errno);
 		}
 
 		free(temp_utf8);
 	}
 
-	pd_printf(PD_ICON_INFO, "Done (deleted %d of %d files)", count_deleted, file_list->length);
+	pd_printf(PD_ICON_INFO, _("Done (deleted %d of %d files)"), count_deleted, file_list->length);
 	pd_end();
 
 	if (count_deleted > 0)
-		fl_refresh();
+		fl_refresh(TRUE);
 }
 
 
@@ -238,8 +255,8 @@ static void rename_file(const char *old_path, const char *new_name)
 
 	if (fu_exists(new_path)) {
 		int button;
-		button = message_box(w_main, "File Exists", 
-				     "A file with this name already exists.\nDo you want to overwrite it?", 
+		button = message_box(w_main, _("File Exists"), 
+				     _("A file with this name already exists.\nDo you want to overwrite it?"), 
 				     0, GTK_STOCK_CANCEL, GTK_STOCK_YES, NULL);
 		if (button == 0) {
 			free(new_path);
@@ -250,15 +267,15 @@ static void rename_file(const char *old_path, const char *new_name)
 	res = rename(old_path, new_path);
 	if (res != 0) {
 		int save_errno = errno;
-		GString *msg = g_string_sized_new(128);
+		GString *msg = g_string_sized_new(256);
 
-		g_string_printf(msg, "Error renaming file:\n%s (errno %d)", strerror(save_errno), save_errno);
-		message_box(w_main, "Error Renaming File", msg->str, 0, GTK_STOCK_OK, NULL);
+		g_string_printf(msg, _("Error renaming file:\n%s (%d)"), strerror(save_errno), save_errno);
+		message_box(w_main, _("Error Renaming File"), msg->str, 0, GTK_STOCK_OK, NULL);
 
 		g_string_free(msg, TRUE);
 	}
 	else {
-		fl_refresh();
+		fl_refresh(TRUE);
 	}
 
 	free(new_path);
@@ -289,21 +306,54 @@ static gboolean check_working_dir()
 
 	res = stat(working_dir->str, &stat_data);
 	if (res < 0) {
-		sb_printf("Error: Can't open directory \"%s\".", working_dir_utf8->str);
+		sb_printf(_("Error: Can't open directory \"%s\"."), working_dir_utf8->str);
 		return FALSE;
 	}
 	if (!S_ISDIR(stat_data.st_mode)) {
-		sb_printf("Error: \"%s\" is not a directory.", working_dir_utf8->str);
+		sb_printf(_("Error: \"%s\" is not a directory."), working_dir_utf8->str);
 		return FALSE;
 	}
 	if (!fu_check_permission(&stat_data, "rx")) {
-		sb_printf("Error: Permission denied for \"%s\".", working_dir_utf8->str);
+		sb_printf(_("Error: Permission denied for \"%s\"."), working_dir_utf8->str);
 		return FALSE;
 	}
 	return TRUE;
 }
 
 
+static void scan_progress_start()
+{
+	progress_dlg_visible = FALSE;
+	chrono_reset(&progress_dlg_time);
+}
+
+static gboolean scan_progress_callback(int dirs, int files)
+{
+	if (!progress_dlg_visible) {
+		if (chrono_get_msec(&progress_dlg_time) > PROGRESS_DLG_DELAY) {
+			progress_dlg_visible = TRUE;
+			spd_display();
+		}
+		else {
+			return FALSE;
+		}
+	}
+
+	spd_update(dirs, files);
+
+	/* flush pending gtk operations so the UI doesn't freeze */
+	while (gtk_events_pending()) gtk_main_iteration();
+
+	return spd_stop_requested();
+}
+
+static void scan_progress_stop()
+{
+	if (progress_dlg_visible)
+		spd_hide();
+}
+
+
 void load_file_list()
 {
 	static const char *patterns[] = {
@@ -321,7 +371,7 @@ void load_file_list()
 	/* clear the form in the edit tab */ 
 	et_unload_file();
 
-	sb_printf("Scanning...");
+	sb_printf(_("Scanning..."));
 	cursor_set_wait();
 
 	/* flush pending operations before we start */
@@ -332,22 +382,25 @@ void load_file_list()
 		g_elist_free_data(file_list);
 		file_list = NULL;
 	}
+	scan_progress_start();
 	file_list = fu_get_file_list(working_dir->str, patterns, 
+				     scan_progress_callback, 
 				     gtk_toggle_button_get_active(cb_recurse),
 				     TRUE);
+	scan_progress_stop();
 
 	/* update the interface */
 	update_tree_view(file_list);
 	count = g_elist_length(file_list);
 	switch (count) {
 	    case 0:
-		sb_printf("No files found.");
+		sb_printf(_("No files found."));
 		break;
 	    case 1:
-		sb_printf("1 file found.");
+		sb_printf(_("1 file found."));
 		break;
 	    default:
-		sb_printf("%d files found.", count);
+		sb_printf(_("%d files found."), count);
 		break;
 	}
 
@@ -395,8 +448,8 @@ void cb_ctx_delete(GtkWidget *widget, GdkEvent *event)
 	if (count == 0)
 		return;
 
-	button = message_box(w_main, "Delete selected files", 
-			     "This will delete the selected files from disk. Proceed?", 
+	button = message_box(w_main, _("Delete selected files"), 
+			     _("This will delete the selected files from disk. Proceed?"), 
 			     0, GTK_STOCK_CANCEL, GTK_STOCK_YES, NULL);
 	if (button == 0)
 		return;
@@ -529,7 +582,7 @@ void cb_wd_changed(GObject *obj, gpointer user_data)
 gboolean cb_wd_keypress(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
 	if (event->keyval == GDK_Return) {
-		fl_refresh();
+		fl_refresh(FALSE);
 		return TRUE;
 	}
 
@@ -560,12 +613,14 @@ void fl_init(GladeXML *xml)
 	lab_file_count = GTK_LABEL(glade_xml_get_widget(xml, "lab_file_count"));
 	menu_file_list = GTK_MENU(glade_xml_get_widget(xml, "menu_file_list"));
 	m_ctx_manual_rename = GTK_MENU_ITEM(glade_xml_get_widget(xml, "m_ctx_manual_rename"));
+	m_ctx_delete = GTK_MENU_ITEM(glade_xml_get_widget(xml, "m_ctx_delete"));
+	m_ctx_unselect_all = GTK_MENU_ITEM(glade_xml_get_widget(xml, "m_ctx_unselect_all"));
 
 	/* 
 	 * create the file chooser
 	 */
 	dlg_wd_select = GTK_DIALOG(gtk_file_chooser_dialog_new(
-					"Select Directory",
+					_("Select Directory"),
 					w_main,
 					GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -725,14 +780,25 @@ const gchar *fl_get_working_dir_utf8()
 }
 
 
-void fl_refresh()
+void fl_refresh(gboolean keep_scroll_pos)
 {
-	const char *ent_wd_text = gtk_entry_get_text(ent_wd);
+	const char *ent_wd_text;
+	GtkAdjustment *adj;
+	double saved_value;
 
+	adj = gtk_tree_view_get_vadjustment(tv_files);
+	saved_value = gtk_adjustment_get_value(adj);
+
+	ent_wd_text = gtk_entry_get_text(ent_wd);
 	if (strcmp(working_dir_utf8->str, ent_wd_text) != 0)
 		fl_set_working_dir_utf8(ent_wd_text);
 	else
 		load_file_list();
+
+	if (keep_scroll_pos) {
+		if (saved_value <= adj->upper)
+			gtk_adjustment_set_value(adj, saved_value);
+	}
 }
 
 
