@@ -3,12 +3,15 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-
-#ifdef HAVE_GNUREGEX_H
-# include <gnuregex.h>	/* used in FreeBSD, I'm told */
-#elif HAVE_REGEX_H
+#ifdef HAVE_GNU_REGEX_H
+# include <gnu/regex.h>	/* for recent FreeBSD */
+#elif HAVE_GNUREGEX_H
+# include <gnuregex.h>	/* for older FreeBSD */
+#else
 # include <regex.h>	/* for everyone else */
 #endif
 
@@ -117,7 +120,7 @@ static parse_info *build_parse_info(const gchar *format)
 	aux_str[j++] = '$';
 	aux_str[j++] = 0;
 
-	/* find the tag fields, record their relative positions and replace 
+	/* find the markers, record their relative positions and replace 
 	   them in the final expression */
 	i = 0;
 	j = 0;
@@ -156,10 +159,10 @@ static parse_info *build_parse_info(const gchar *format)
 				i += 9;
 				j += 4;
 			} else if (strncmp(&aux_str[i], "<track>", 7) == 0) {
-				strncpy(&expr_str[j], "([0-9]*)", 8);
+				strncpy(&expr_str[j], "(.*)", 4);
 				info->track = pos++;
 				i += 7;
-				j += 8;
+				j += 4;
 			} else if (strncmp(&aux_str[i], "<genre>", 7) == 0) {
 				strncpy(&expr_str[j], "(.*)", 4);
 				info->genre = pos++;
@@ -316,15 +319,15 @@ static void tag_files(GEList *file_list)
 	gboolean write_genre   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cb_genre));
 	gboolean track_auto    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cb_track_auto));
 
-	pd_start("Tagging Files");
+	pd_start(_("Tagging Files"));
 	if (!(write_title || write_artist || write_album || write_year || 
 	      write_comment || write_track || write_genre))
 	{
-		pd_printf(PD_ICON_INFO, "No tag fields to set!");
+		pd_printf(PD_ICON_INFO, _("No tag fields to set!"));
 		pd_end();
 		return;
 	}
-	pd_printf(PD_ICON_INFO, "Starting in directory \"%s\"", fl_get_working_dir_utf8());
+	pd_printf(PD_ICON_INFO, _("Starting in directory \"%s\""), fl_get_working_dir_utf8());
 
 	if (*use_filename) {
 		const gchar *format = gtk_entry_get_text(ent_tag_format);
@@ -345,7 +348,7 @@ static void tag_files(GEList *file_list)
 		pd_scroll_to_bottom();
 		while (gtk_events_pending()) gtk_main_iteration();
 		if (pd_stop_requested()) {
-			pd_printf(PD_ICON_WARN, "Operation stopped at user's request");
+			pd_printf(PD_ICON_WARN, _("Operation stopped at user's request"));
 			break;
 		}
 
@@ -355,9 +358,9 @@ static void tag_files(GEList *file_list)
 
 		if (!fu_compare_file_paths(last_path, curr_path)) {
 			gchar *p;
-			temp_utf8 = str_filename_to_utf8(curr_path, "(UTF8 conversion error)");
+			temp_utf8 = str_filename_to_utf8(curr_path, _("(UTF8 conversion error)"));
 			p = g_utf8_strrchr(temp_utf8, -1, '/');
-			pd_printf(PD_ICON_INFO, "Entering directory \"%.*s\"", (gint)(p-temp_utf8), temp_utf8);
+			pd_printf(PD_ICON_INFO, _("Entering directory \"%.*s\""), (gint)(p-temp_utf8), temp_utf8);
 			free(temp_utf8);
 
 			track_auto_index = 1;	/* new dir, reset track auto-increment */
@@ -365,18 +368,18 @@ static void tag_files(GEList *file_list)
 		last_path = curr_path;
 
 		file_name = fu_last_n_path_components(curr_path, 1);
-		file_name_utf8 = str_filename_to_utf8(file_name, "(UTF8 conversion error)");
+		file_name_utf8 = str_filename_to_utf8(file_name, _("(UTF8 conversion error)"));
 
 		res = audio_file_new(&af, curr_path, TRUE);
 		if (res != AF_OK) {
-			pd_printf(PD_ICON_FAIL, "Error tagging \"%s\"", file_name_utf8);
+			pd_printf(PD_ICON_FAIL, _("Error tagging \"%s\""), file_name_utf8);
 
 			if (res == AF_ERR_FILE)
-				pd_printf(PD_ICON_NONE, "Couldn't open file for writing");
+				pd_printf(PD_ICON_NONE, _("Couldn't open file for writing"));
 			else if (res == AF_ERR_FORMAT)
-				pd_printf(PD_ICON_NONE, "Audio format not recognized");
+				pd_printf(PD_ICON_NONE, _("Audio format not recognized"));
 			else 
-				pd_printf(PD_ICON_NONE, "Unknown error (%d)", res);
+				pd_printf(PD_ICON_NONE, _("Unknown error (%d)"), res);
 
 			goto _continue;
 		}
@@ -415,8 +418,8 @@ static void tag_files(GEList *file_list)
 			free(full_path);
 
 			if (!res) {
-				pd_printf(PD_ICON_FAIL, "Error tagging \"%s\"", file_name_utf8);
-				pd_printf(PD_ICON_NONE, "File name does not match expected format");
+				pd_printf(PD_ICON_FAIL, _("Error tagging \"%s\""), file_name_utf8);
+				pd_printf(PD_ICON_NONE, _("File name does not match expected format"));
 				goto _continue;
 			}
 		}
@@ -425,17 +428,17 @@ static void tag_files(GEList *file_list)
 		res = audio_file_write_changes(af);
 		if (res != AF_OK) {
 			save_errno = errno;
-			pd_printf(PD_ICON_FAIL, "Error tagging \"%s\"", file_name_utf8);
+			pd_printf(PD_ICON_FAIL, _("Error tagging \"%s\""), file_name_utf8);
 
 			if (res == AF_ERR_FILE)
-				pd_printf(PD_ICON_NONE, "%s (errno %d)", strerror(save_errno), save_errno);
+				pd_printf(PD_ICON_NONE, "%s (%d)", strerror(save_errno), save_errno);
 			else 
-				pd_printf(PD_ICON_NONE, "Unknown error (%d)", res);
+				pd_printf(PD_ICON_NONE, _("Unknown error (%d)"), res);
 
 			goto _continue;
 		}
 
-		pd_printf(PD_ICON_OK, "Tagged \"%s\"", file_name_utf8);
+		pd_printf(PD_ICON_OK, _("Tagged \"%s\""), file_name_utf8);
 		count_tagged++;
 
 	_continue:
@@ -446,7 +449,7 @@ static void tag_files(GEList *file_list)
 		}
 	}
 
-	pd_printf(PD_ICON_INFO, "Done (tagged %d of %d files)", count_tagged, count_total);
+	pd_printf(PD_ICON_INFO, _("Done (tagged %d of %d files)"), count_tagged, count_total);
 	pd_end();
 
 	if (pi != NULL)
@@ -465,7 +468,7 @@ static void start_operation()
 
 	if (g_elist_length(file_list) == 0) {
 		pd_start(NULL);
-		pd_printf(PD_ICON_FAIL, "No files selected");
+		pd_printf(PD_ICON_FAIL, _("No files selected"));
 		pd_end();
 
 		g_elist_free(file_list);
